@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2020 eGauge Systems LLC
+# Copyright (c) 2020-2021 eGauge Systems LLC
 #       1644 Conestoga St, Suite 2
 #       Boulder, CO 80301
 #       voice: 720-545-9767
@@ -29,6 +29,8 @@
 #
 from PySide2.QtWidgets import QMessageBox, QWidget
 
+import egauge.ctid as ctid
+
 def invalid_double(string):
     try:
         float(string)
@@ -168,28 +170,54 @@ class CT(Sensor_Params):
             cal_table[str(levels[row])] = (v_cal, p_cal)
         template['cal_table'] = cal_table
 
-class Volt(Sensor_Params):
+class Linear(Sensor_Params):
+    def __init__(self, ui, parent, title):
+        super().__init__(ui, parent, title)
+        max_float = 3.4028235e38
+        self.ui.scale_spinbox.setRange(-max_float, max_float)
+        self.ui.offset_spinbox.setRange(-max_float, max_float)
+        for _, unit_desc in ctid.SENSOR_UNITS:
+            self.ui.unit_comboBox.addItem(unit_desc)
+        self.ui.unit_comboBox.currentIndexChanged.connect(self._unit_changed)
+
+    def _unit_changed(self):
+        unit_code = self.ui.unit_comboBox.currentIndex()
+        unit = ctid.get_sensor_unit(unit_code)
+        if unit == '':
+            scale_unit = ' 1/V'
+            offset_unit = ''
+        else:
+            scale_unit = ' %s/V' % unit
+            offset_unit = ' %s' % unit
+        self.ui.scale_spinbox.setSuffix(scale_unit)
+        self.ui.offset_spinbox.setSuffix(offset_unit)
+
     def set_input_enabled(self, enabled):
+        self.ui.unit_comboBox.setEnabled(enabled)
         self.ui.scale_spinbox.setEnabled(enabled)
         self.ui.offset_spinbox.setEnabled(enabled)
         self.ui.delay_spinbox.setEnabled(enabled)
 
     def load(self, template):
+        self.ui.unit_comboBox.setCurrentIndex(template['unit'])
         self.ui.scale_spinbox.setValue(template['scale'])
         self.ui.offset_spinbox.setValue(template['offset'])
         self.ui.delay_spinbox.setValue(template['delay'])
 
     def save(self, template):
+        template['unit'] = self.ui.unit_comboBox.currentIndex()
         template['scale'] = self.ui.scale_spinbox.value()
         template['offset'] = self.ui.offset_spinbox.value()
         template['delay'] = self.ui.delay_spinbox.value()
 
     def encoder_argv(self, template):
-        return ['--scale', str(template['scale']),
+        return ['--unit', str(template['unit']),
+                '--scale', str(template['scale']),
                 '--offset', str(template['offset']),
                 '--delay', str(template['delay'])]
 
     def table_to_template(self, table, template):
+        template['unit'] = table.sensor_unit
         template['scale'] = table.scale
         template['offset'] = table.offset
         template['delay'] = table.delay
